/*
OldMcData - The Data Farmer
Copyright (C) 2005 Referentia Systems, Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*/

package oldmcdata.analyzers;


import java.io.File;
import java.util.List;

import oldmcdata.analyzers.utils.FileWatcher;
import oldmcdata.analyzers.utils.FitnessCalculator;
import oldmcdata.analyzers.utils.FitnessCalculatorFactory;
import oldmcdata.analyzers.utils.SimpleFitnessCalculator;
import oldmcdata.analyzers.utils.model.ModelDataProcessor;
import oldmcdata.utils.StudyXML;

import org.dom4j.Element;

/**
 * Title:
 * Description:
 *Copyright: 2000 - 2005
 *  
 * @author Stephen C. Upton
 * @version 1.0
 */

public class SimpleFitnessAnalyzer extends AbstractAnalyzer {
    private final static String FITNESS = "Fitness",
    MOENAME = "MOEName",
    MOEPOSITION = "MOEPosition",
    STATISTIC = "Statistic",
    LINESTOSKIP = "LinesToSkip",
    FITNESSCALCULATOR = "FitnessCalculator";
    private String moe, moePosition, statistic, linesToSkip;
    private File moeDir, studyDir;
    private String moeFileStub, basecaseFileName;
    private ModelDataProcessor modelDataProcessor;
    private Element iterationElement;
    private FitnessCalculator fitnessCalculator;

    public SimpleFitnessAnalyzer() {
        fitnessCalculator = new SimpleFitnessCalculator();
    }

    public void initialize() {
        System.out.println(this.getClass().getName() + ":initialize()");
        try {
            configure();
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(-1);
        }
    }

    private void configure() {
        studyXMLHandler.openStudyXMLDocument();
        Element parameters = studyXMLHandler.getAnalyzerAlgorithmElement().element(StudyXML.PARAMETERS);
        Element fitness = parameters.element(FITNESS);
        if (fitness == null)
            throw new RuntimeException("A Fitness must be specified for this Analyzer");
        setFitnessMOE(fitness.attributeValue(MOENAME));
        setFitnessStatistic(fitness.attributeValue(STATISTIC));
        setFitnessMOEPosition(fitness.attributeValue(MOEPOSITION));
        setLinesToSkip(fitness.attributeValue(LINESTOSKIP));
        Element fitnessCalcElement = parameters.element(FITNESSCALCULATOR);
        if (fitnessCalcElement != null) // else use default SimpleFitnessCalculator
            setFitnessCalculator(fitnessCalcElement.getTextTrim());
        fitnessCalculator.setStatistic(statistic);
        if (moePosition != null) {
            modelDataProcessor = new ModelDataProcessor();
            modelDataProcessor.setMOEPosition(moePosition);
            modelDataProcessor.setLinesToSkip(linesToSkip);
        } else
            modelDataProcessor = ModelDataProcessor.getInstance(studyXMLHandler.getModelName(),
                    studyXMLHandler.getModelVersion());
        modelDataProcessor.setConfigFile(configFile);
        modelDataProcessor.setMOE(moe);
    }

    public void setFitnessMOE(String noe) {
        if (noe == null)
            throw new RuntimeException("Fitness must specify a MOE");
        moe = noe;
    }

    public void setLinesToSkip(String lines) {
    	// if null, this will default to 0 linesToSkip in the ModelDataProcessor
    	linesToSkip = lines;
    }
    
    public void setFitnessMOEPosition(String pos) {
        moePosition = pos;
    }

    public void setFitnessStatistic(String noe) {
        if (noe == null)
            throw new RuntimeException("Fitness must specify a Statistic");
        statistic = noe;
    }

    public void setFitnessCalculator(String calc) {
        if (calc != null) {
            fitnessCalculator = FitnessCalculatorFactory.getInstance(calc);
        }
        // else use default of SimpleFitnessCalculator - set in constructor
    }


    public boolean analyze() {
        System.out.println(this.getClass().getName() + ":analyze()");
        studyXMLHandler.openStudyXMLDocument();
        if (!checkOutputFiles()) return false;
        processExcursions();
        studyXMLHandler.writeStudyXMLDocument();
        return true;
    }

    private boolean checkOutputFiles() {
        FileWatcher fileWatcher = new FileWatcher();
        fileWatcher.setMoeFilesToWatch(studyXMLHandler.getMOEFileNames());
        return fileWatcher.checkForOutput();
    }

    private void processExcursions() {
        iterationElement = studyXMLHandler.getMostRecentIterationElement();
        constructFileInfo();
        modelDataProcessor.setOutputFileInfo(moeDir, moeFileStub);
        modelDataProcessor.setBasecaseFileInfo(studyDir, basecaseFileName);
        modelDataProcessor.buildSchema();
        modelDataProcessor.setNumberOfReplications(studyXMLHandler.getNumberReplicates());

        List excursions = iterationElement.elements(StudyXML.EXCURSION);
        for (int i = 0; i < excursions.size(); i++) {
            processExcursion((Element) excursions.get(i));
        }

    }

    private void constructFileInfo() {
        studyDir = studyXMLHandler.getStudyDirectory();
        Element excursionFileInfo = iterationElement.element(StudyXML.EXCURSIONFILEINFO);
        basecaseFileName = excursionFileInfo.element(StudyXML.BASECASEFILENAME).getTextTrim();
        String moeDirName = excursionFileInfo.element(StudyXML.MOEDIR).getTextTrim();
        moeDir = new File(studyDir, moeDirName);
        moeFileStub = excursionFileInfo.element(StudyXML.MOEFILESTUB).getTextTrim();
    }


    private void processExcursion(Element excursion) {
        String excursionNumber = excursion.attributeValue(StudyXML.ID);
        System.out.println("Analyzing excursion: " + excursionNumber);
        Element fitness = excursion.addElement(FITNESS);
        double[] data = modelDataProcessor.getData(excursionNumber);
        if ( data == null ){
        	fitness.setText("NaN");
        	return;
        }
        printdata(data, excursionNumber);
        Double realFitness = fitnessCalculator.calculateFitness(data);
        fitness.setText(realFitness.toString());
    }

    private void printdata(double[] data, String excursionNumber) {
        System.out.println("data for excursionNumber: " + excursionNumber);
        for (int i = 0; i < data.length; i++)
            System.out.print(i + ":" + data[i] + ",");
        System.out.println();
    }

}
