/**
 * 
 */
package oldmcdata.evaluators.condor;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.EnumMap;
import java.util.List;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;

/**
 * @author stephen upton
 * @date 12/21/07
 * 
 * Creates a condor submit file using a template as the basis.
 */

enum CondorEntry { UNIVERSE("universe"), GETENV("getenv"), REQUIREMENTS("requirements"), RANK("Rank"),
	ENVIRONMENT("environment"), EXECUTABLE("executable"), ARGUMENTS("arguments"), 
	TRANSFER_INPUT_FILES("transfer_input_files"),
	INITIALDIR("initialdir"), LOG("log"), OUTPUT("output"), ERROR("error"), 
	NOTIFICATION("notification"), QUEUE("queue");
	private String value;
	private CondorEntry(String value) {
		this.value = value;
	}
	public String getValue() { return value; }
}

public class CondorSubmitFileFromTemplate {
    private final static String space = " ",
    							DEFAULTENTRYSEPARATOR = "=";
	private String condorTemplateFileName;
    private String lineSeparator;
    private List<String> condorLines;
    private File submitFile;
    private File directoryToWriteSubmitFiles;
    private Element rootElement;
    private EnumMap<CondorEntry, CondorEntryHandler> condorEntries;
	
	public CondorSubmitFileFromTemplate(String condorTemplate) {
		condorTemplateFileName = condorTemplate;
        lineSeparator = System.getProperty("line.separator");
        condorLines = new ArrayList<String>(); 
        directoryToWriteSubmitFiles = new File(System.getProperty("user.dir"));
        condorEntries = new EnumMap<CondorEntry, CondorEntryHandler>(CondorEntry.class);
        //initialize all the entries with CondorEntryHandler
        for ( CondorEntry  e : CondorEntry.values() ) {
        	condorEntries.put(e, new CondorEntryHandler());
        }
        
        //add the special CondorEntryHandlers
        condorEntries.put(CondorEntry.ARGUMENTS, new ArgumentEntryHandler());
        condorEntries.put(CondorEntry.TRANSFER_INPUT_FILES, new TransferInputFilesEntryHandler() );
	}
	
	public void createSubmitFiles() {
		openCondorTemplate();
		createOutputDirectory();
		
		for (int rep=1; rep<=4; rep++)
			createSubmitFile(rep);


	}
	
	public void createSubmitFile(int i) {
		condorLines.clear();
		Element condorElement = rootElement.element("condor");
        for ( CondorEntry  e : condorEntries.keySet() ) {
        	CondorEntryHandler handler = condorEntries.get(e);
        	
        	String s = handler.processEntry(condorElement.element(e.getValue()), i);
        	condorLines.add(s);
        	System.out.println("entry: " + e.toString() + " value: " + s);
        }
		String submitFileName = "submit-" + i + ".dat";
		submitFile = new File(directoryToWriteSubmitFiles, submitFileName);
		writeSubmitFile();
	}
	
	public String getOutputDirectory() {
		return directoryToWriteSubmitFiles.getAbsolutePath().toString();
	}
	
	private void createOutputDirectory() {
		String directory = rootElement.elementTextTrim("directory");
		if ( directory == null || directory.equalsIgnoreCase("this") || directory.equalsIgnoreCase(".")
				|| directory.length() == 0) {
			System.out.println("Directory: " + directoryToWriteSubmitFiles.toString());
			return; //use current directory, user.dir
		}
		directoryToWriteSubmitFiles = new File(directory);
		if ( !directoryToWriteSubmitFiles.exists() ) {
			directoryToWriteSubmitFiles.mkdir();
		}
		
		System.out.println("Directory: " + directoryToWriteSubmitFiles.toString());
	}
	
	public void openCondorTemplate() {
        try {
            SAXReader xmlReader = new SAXReader();
            xmlReader.setStripWhitespaceText(true);
            Document studyDoc = xmlReader.read(condorTemplateFileName);
            rootElement = studyDoc.getRootElement();
        } 
            catch (DocumentException de) {
            System.out.println("Error attempting to parse: " + condorTemplateFileName);
            de.printStackTrace(System.out);
            System.exit(-1);
        }
	}
	
    public void writeSubmitFile() {
        try {
            FileWriter fw = new FileWriter(submitFile);
            for (int i = 0; i < condorLines.size(); i++) {
                String condorLine = (String) condorLines.get(i);
                fw.write(condorLine);
                fw.write(lineSeparator);
            }
            fw.close();
        } catch (IOException ioe) {
            System.out.println("Error attempting to open: " + submitFile);
            ioe.printStackTrace(System.out);
            System.exit(-1);
        }
    }
	
	public static void main(String[] args) {
		String condorTemplate = args[0];
		System.out.println("Starting file creation...");
		CondorSubmitFileFromTemplate cft = new CondorSubmitFileFromTemplate(condorTemplate);

		cft.createSubmitFiles();
		System.out.println("Creating files in directory: " + cft.getOutputDirectory());
		System.out.println("....Completed file creation");
	}
}

class CondorEntryHandler {
	protected String argsep = " = ";
	
	public String processEntry(Element e, int i) {
		StringBuilder s = new StringBuilder(e.getName());
		s.append(addArgSep(e));
		
		s.append(e.getTextTrim());
		
		s.append(addIteration(e, i));
		
		s.append(addSuffix(e));
		
		return s.toString();
	}
	
	protected String addArgSep(Element e) {
		String argsepValue = e.attributeValue("argsep");
		if( argsepValue != null ) {
			return argsepValue;
		}
		else
			return argsep;
	}
	
	protected String addIteration(Element e, int i) {
		String iteration = e.attributeValue("append");
		if ( iteration != null )
			return Integer.toString(i);
		
		return "";
	}
	
	protected String addSuffix(Element e) {
		String suffix = e.attributeValue("suffix");
		if ( suffix != null )
			return suffix;
		
		return "";
	}
}

class ArgumentEntryHandler extends CondorEntryHandler {
	private String argumentSep = " ";
	
	@SuppressWarnings("unchecked")
	public String processEntry(Element e, int i) {
		StringBuilder s = new StringBuilder(e.getName());
		s.append(addArgSep(e));
		List<Element> argumentList = e.elements("argument");
		for (Element argument : argumentList ) {
			s.append(argument.getTextTrim());
			s.append(addIteration(argument,i));
			s.append(addSuffix(argument));
			s.append(argumentSep);
		}
		
		s.deleteCharAt(s.length() -1 );
		
		return s.toString();
	}
}

class TransferInputFilesEntryHandler extends CondorEntryHandler {
	private String argumentSep = ",";
	
	@SuppressWarnings("unchecked")
	public String processEntry(Element e, int i) {
		StringBuilder s = new StringBuilder(e.getName());
		s.append(addArgSep(e));
		List<Element> fileList = e.elements("file");
		for (Element file : fileList ) {
			s.append(file.getTextTrim());
			s.append(addIteration(file,i));
			s.append(addSuffix(file));
			s.append(argumentSep);
		}
		s.deleteCharAt(s.length() -1 );
		
		return s.toString();
	}
}

