/*
 * Created on May 7, 2004
 *
 */
package oldmcdata.postprocessors;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
/**
 *Copyright: 2000 - 2005
 *   Referentia Systems, Inc.
 * @author Stephen C. Upton
 *
 */
public class ColumnSelector {
	private PrintWriter logFileWriter;
	private String delimiter = ",";
	private String[] header = null;
	private Set columnNumbers;
	private int numberOfColumns = 0;
	private String columnNumbersString,startsWithString,containsString;
	private Pattern pattern;
	private PostProcessTask myTask;
		
	public ColumnSelector(PostProcessTask task) {
		myTask = task;
		this.configure();
	}
	
	private void configure() {
		logFileWriter = myTask.getLogFileWriter();
		columnNumbers = new TreeSet();
		Map selectColumnsMap = myTask.getDataMap(PostProcessTask.SELECTCOLUMNS);
		this.setNumberString((String)selectColumnsMap.get(PostProcessTask.NUMBER));
		this.setContainsString((String)selectColumnsMap.get(PostProcessTask.CONTAINS));
		this.setStartsWithString((String)selectColumnsMap.get(PostProcessTask.STARTSWITH));
		this.setPattern((Pattern)selectColumnsMap.get(PostProcessTask.PATTERN));

	}
	
	/* Determines which columns the user wants based on the union of the set of 
	 * numbers calculated from:
	 * columnNumbers, startsWithString, contains, and the regular expression pattern
	 * using the first row of data as the basis.
	 * The number of columns is first calculated by parsing the first row using the delimiter.
	 * The column number is based on using 1 as the first column.
	 */
	public Set determineColumns(String headerRow) {
		columnNumbers.clear();
		header = headerRow.split(delimiter);	
		numberOfColumns = header.length;	
		processColumnNumbersString();
		processStartsWithString();
		processContainsString();
		processRegexPattern();
		
		return columnNumbers;
	}
	
	public void setNumberString(String s) {
		columnNumbersString = s;
	}
	
	public void setStartsWithString(String s) {
		startsWithString = s;		
	}
	
	public void setContainsString(String s) {
		containsString = s;
	}
	
	public void setPattern(Pattern p) {
		pattern = p;
	}

	
	/* Splits the column number string using the delimiter. Uses only those strings
	 * that are integers and ignores the rest. Then, ignores those integers that
	 * are not within the range of the [1,numberOfColumns].
	 */
	private void processColumnNumbersString() {
		// process the default of All columns if no SelectColumns element was specified
	    if ( columnNumbersString.equalsIgnoreCase("All") ) {
	        logFileWriter.println("Processing the default of All columns");
	        for (int i=0;i<numberOfColumns;i++) {
	            Integer columnIndex = new Integer(i+1);
	            columnNumbers.add(columnIndex);
	        }
	        
		    return;
		}
			
		List processedColumns = new ArrayList();
		
		String[] numbers = columnNumbersString.split(delimiter);
		for (int i=0;i<numbers.length;i++) {
			String numberToTry = numbers[i];
			try {
				Integer columnIndex = new Integer(numberToTry);
				if ( columnIndex.intValue() > 0 && columnIndex.intValue() <= numberOfColumns) {
					processedColumns.add(columnIndex.toString());
					columnNumbers.add(columnIndex);
				}
				else
					logFileWriter.println("columnNumber outside of range: " + columnIndex.toString());
			}
			catch(NumberFormatException nfe) {
				logFileWriter.println("Unrecognized number in ColumnNumber: " + numberToTry);
			}
			
		}
		printProcessedColumns("ColumnNumbers", processedColumns);
	}
	
	/* Determines column numbers by matching the header row entries with the
	 * startsWith string. If there is a match, then use the associated column number.
	 */
	private void processStartsWithString() {
		if ( startsWithString == null || startsWithString.length() == 0 )
			return;
		List processedColumns = new ArrayList();
		
		String[] starterStrings = startsWithString.split(delimiter);
		for (int j=0;j<starterStrings.length;j++) {
			for (int i=0; i<numberOfColumns;i++) {
				if ( header[i].startsWith(starterStrings[j]) ) {
					Integer addColumnNumber = new Integer(i+1);
					processedColumns.add(addColumnNumber.toString());
					columnNumbers.add(addColumnNumber);
				}
			}
		}

		printProcessedColumns("StartsWithString", processedColumns);		
	}
	
	/* Determines column numbers by matching the header row entries with the
	 * containsString. If the header row entry contains the containsString, 
	 * then use the associated column number.
	 */
	private void processContainsString(){
		if ( containsString == null || containsString.length() == 0 )
			return;
		List processedColumns = new ArrayList();
		
		String[] containStrings = containsString.split(delimiter);	
		for (int j=0;j<containStrings.length;j++) {	
			for (int i=0; i<numberOfColumns;i++) {
				if ( header[i].indexOf(containStrings[j]) != -1 ) {
					Integer addColumnNumber = new Integer(i+1);
					processedColumns.add(addColumnNumber.toString());
					columnNumbers.add(new Integer(i+1));
				}
			}
		}
		printProcessedColumns("ContainsString", processedColumns);		

	}
	
	private void processRegexPattern() {
		if ( pattern == null )
			return;
		List processedColumns = new ArrayList();
		Matcher matcher;
		
		for (int i=0; i<numberOfColumns;i++) {
			matcher = pattern.matcher(header[i]);
			if ( matcher.find() ) {
				Integer addColumnNumber = new Integer(i+1);
				processedColumns.add(addColumnNumber.toString());
				columnNumbers.add(addColumnNumber);
				matcher.reset();
			}
		}	
		
		printProcessedColumns("RegexPattern", processedColumns);	
	}
	
	/* Prints the column numbers associated with the method doing the processing.
	 * For debugging purposes.
	 * 
	 */
	private void printProcessedColumns(String methodProcessing, List processedColumns) {
		String columnsProcessed = convertListToString(processedColumns);
		logFileWriter.println("Column Numbers calculated by: " + methodProcessing);
		logFileWriter.println("Column Numbers: " + columnsProcessed);
	}
	
	/* Convenience function to convert the list of strings to a single string.
	 * 
	 */
	private String convertListToString(List listOfStrings) {
		StringBuffer sb = new StringBuffer();
		if ( listOfStrings.size() == 0 )
			return sb.toString(); // empty string		
		for(Iterator stringList = listOfStrings.iterator(); stringList.hasNext(); ) {
			sb.append((String)stringList.next());
			sb.append(delimiter);
		}

		// remove the last delimiter
		sb.deleteCharAt(sb.length() - 1);
		
		return sb.toString();
	}
	

}
