package oldmcdata.postprocessors;

import java.io.PrintWriter;
import java.util.Map;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import org.dom4j.Element;

/** To be valid, the PostProcessTask element must have
 * a FilePattern element, a ReplicatePattern element, and an InFile element. The FilePattern and ReplicatePattern 
 * elements specifies the pattern to match following the rules in the java.util.regex.Pattern 
 * class documentation. 
 * The InFile element has 4 elements:
 * a SkipLines element, which describes which lines to skip in the file, such as headers or comment lines,
 * or rows that you do not want to consider, 
 * a SelectColumns element, which specifies how to select columns, 
 * a ColumnNames element, which is either True or False depending on whether the user specifies 
 * the first row not skipped as a "header" row, i.e., what the names of the columns are,
 * a RowNames element, which is either True or False depending on whether the user designates the
 * first column selected is used as the names for each subsequent row selected.
 * 
 * Each of the SkipLines and SelectColumns elements can have 4 other elements
 * that specify which items to skip or select, respectively. They are:
 * a StartsWith element, which specifies a starting string,
 * a Contains element, which is a (sub)string that matches if that item (row/column)
 * contains the resulting string,
 * a Number element, which is either a single number, representing that many lines to skip or that
 * particular column, or a string of digits to select
 * a Pattern element, which is a regular expression to allow more elaborate matching of skipped
 * lines or columns. 
 * 
 *Copyright: 2000 - 2005
 *   Referentia Systems, Inc.
 * @author Stephen C. Upton
 *
 */	

public class PostProcessTaskValidator {

	private PostProcessTask task;
	private PrintWriter logFileWriter;
	private Element taskElement, inFileElement,skipLinesElement, selectColumnsElement,
				selectRowsElement, outFileElement;
	
	public PostProcessTaskValidator(PostProcessTask task) {
		this.task = task;
		this.taskElement = task.getTaskElement();
		this.logFileWriter = task.getLogFileWriter();
	}
	
	public boolean validate() {
		processTaskName();
		processFileDirectory();
		if ( !validateFilePattern() ) return false;
		if ( !validateReplicatePattern() ) return false;
		if ( !validateInFile() ) return false;
		if ( !validateOutFile() ) return false;
		setColumnRowNames();
		
		return true;
	}
	
	private void processTaskName() {
		Element taskNameElement = taskElement.element(PostProcessTask.TASKNAME);
		if ( taskNameElement == null )
			task.setTaskName("Task");
		else
			task.setTaskName(taskNameElement.getTextTrim());
	}
	
	private void processFileDirectory() {
		Element fileDirectory = taskElement.element(PostProcessTask.FILEDIRECTORY);
		if ( fileDirectory == null )
			task.setFileDirectory(".");
		else
			task.setFileDirectory(fileDirectory.getTextTrim());
	}

	private boolean validateFilePattern() {
		String filePatternString = taskElement.elementText(PostProcessTask.FILEPATTERN);
		// if there is no file pattern to match, then no files to process!
		if ( filePatternString == null ) {
			logFileWriter.println("ERROR: No file pattern to match");
			return false;
		} 
		Pattern filePattern = compilePattern(filePatternString, PostProcessTask.FILEPATTERN);
		if ( filePattern == null )
			return false;
		
		task.setFilePattern(filePattern);
		return true;			
	}
	
	private boolean validateReplicatePattern() {
		String replicatePatternString  = taskElement.elementTextTrim(PostProcessTask.REPLICATEPATTERN); 
		if ( replicatePatternString == null ) {
			logFileWriter.println("ERROR: No replicate pattern to match");
			return false;
		} 		
		Pattern replicatePattern = compilePattern(replicatePatternString, PostProcessTask.REPLICATEPATTERN);
		if ( replicatePattern == null )
			return false;
		
		task.setReplicatePattern(replicatePattern);
		return true;
	}
	
	private Pattern compilePattern(String patternString, String elementName) {
		String errorMessage = "ERROR: Pattern Exception using pattern: " + patternString 
		+ "in " + elementName;
		Pattern pattern = null;
		try {
			pattern = Pattern.compile(patternString);
			
		} catch(PatternSyntaxException pse) {
			logFileWriter.println(errorMessage);
			System.out.println(errorMessage);
			logFileWriter.println(pse.getMessage());
		}	
		return pattern;		
		
		
	}		
	
	private boolean validateInFile() {
		inFileElement = taskElement.element(PostProcessTask.INFILE);
		if ( inFileElement == null ) {
			logFileWriter.println("ERROR: No InFile element to process");
			return false;
		}	
		

		String skipLines = PostProcessTask.SKIPLINES;
		skipLinesElement = inFileElement.element(skipLines);
		Map skipLinesMap = task.getDataMap(skipLines);
		if ( !validateElement(skipLinesElement,skipLines, skipLinesMap) ) 
			return false;
		
		String selectColumns = PostProcessTask.SELECTCOLUMNS;
		selectColumnsElement = inFileElement.element(selectColumns);
		Map selectColumnsMap = task.getDataMap(selectColumns);
		if ( selectColumnsElement == null ) {
			logFileWriter.println("NOTICE: No " + selectColumns + " entry; " +
			"will continue to process using defaults");
			selectColumnsMap.put(PostProcessTask.NUMBER, "All");
		}
		else 
		    if ( !validateElement(selectColumnsElement, selectColumns, selectColumnsMap) ) 
		        return false;

		String selectRows = PostProcessTask.SELECTROWS;
		selectRowsElement = inFileElement.element(selectRows);
		Map selectRowsMap = task.getDataMap(selectRows);
		if ( selectRowsElement == null ) {
			logFileWriter.println("NOTICE: No " + selectRows + " entry; " +
			"will continue to process using defaults");
		}
		else 
		    if ( !validateElement(selectRowsElement, selectRows, selectRowsMap) ) 
				return false;			

		processExtraInfoElement();
		
		return true;			
		
	}
	
	private void processExtraInfoElement() {
	    String extraInfo = PostProcessTask.EXTRAINFO;
	    Element extraInfoElement = inFileElement.element(extraInfo);
	    if ( extraInfoElement == null ) 
	        return;
	    Map extraInfoMap = task.getDataMap(PostProcessTask.EXTRAINFO);
	    logFileWriter.println("Found ExtraInfo element to process");
	    
		String startsWith = extraInfoElement.elementTextTrim(PostProcessTask.STARTSWITH);
		if ( startsWith != null ) 
		    extraInfoMap.put(PostProcessTask.STARTSWITH, startsWith);
		else {
		    extraInfoMap.put(PostProcessTask.STARTSWITH, null);
		    logFileWriter.println("NOTICE: The ExtraInfo element is incomplete " + 
		            	" No STARTSWITH element");
		    return;
		}
		
		String headerName = extraInfoElement.elementTextTrim(PostProcessTask.HEADERNAME);
		if ( headerName != null ) 
		    extraInfoMap.put(PostProcessTask.HEADERNAME, headerName);
		else
		    extraInfoMap.put(PostProcessTask.HEADERNAME, "NoName");
			
		String pattern = extraInfoElement.elementTextTrim(PostProcessTask.PATTERN);
		if ( pattern != null ) {
			Pattern p = compilePattern(pattern, extraInfo);
			extraInfoMap.put(PostProcessTask.PATTERN, p);
		}
		else {
		    logFileWriter.println("NOTICE: The ExtraInfo element is incomplete " + 
        	" No PATTERN element");
			// then, clear the map since it is incomplete
			extraInfoMap.clear();
		}
	}
	

	private boolean validateElement(Element element, String elementName, Map valueMap) {
		boolean atLeastOneEntry = false;
		String startsWith = element.elementTextTrim(PostProcessTask.STARTSWITH);
		if ( startsWith != null ) {
			valueMap.put(PostProcessTask.STARTSWITH, startsWith);
			atLeastOneEntry = true; 
		}
		
		String contains = element.elementTextTrim(PostProcessTask.CONTAINS);
		if ( contains != null ) {
			valueMap.put(PostProcessTask.CONTAINS, contains);
			atLeastOneEntry = true; 
		}	
		
		String number = element.elementTextTrim(PostProcessTask.NUMBER);
		if ( number != null ) {
			valueMap.put(PostProcessTask.NUMBER, number);
			atLeastOneEntry = true; 
		}
		
		String pattern = element.elementTextTrim(PostProcessTask.PATTERN);
		if ( pattern != null ) {
			Pattern p = compilePattern(pattern, elementName);
			valueMap.put(PostProcessTask.PATTERN, p);
			atLeastOneEntry = true; 
		}
		
		if ( !atLeastOneEntry )
			logFileWriter.println("NOTICE: There is a " + elementName + " entry but no entries" +
					" to specify which lines to process; therefore none will be processed according" +
			"to this element");	
		
		return true;
	}
	
	private void setColumnRowNames() {
		// if header row use is not specified, then assume true and use first row
		Element headerRowElement = inFileElement.element(PostProcessTask.HEADERROW);
		if ( headerRowElement == null )
			task.setUseHeaderRow(true);
		else {
			String headerRow = headerRowElement.getTextTrim();
			if ( headerRow.equalsIgnoreCase("F"))
				task.setUseHeaderRow(false);
		}
		// if column names not specified, then assume true and first row are the names
		Element columnNamesElement = inFileElement.element(PostProcessTask.COLUMNNAMES);
		if ( columnNamesElement == null )
			task.setUseColumnNames(true);
		else {
			String columnNames = columnNamesElement.getTextTrim();
			if ( columnNames.equalsIgnoreCase("F"))
				task.setUseColumnNames(false);
		}
		
		// if row names is not specified, then assume false
		// if true, then first column is used as the row names
		Element rowNamesElement = inFileElement.element(PostProcessTask.ROWNAMES);
		if ( rowNamesElement == null )
			task.setUseRowNames(false);
		else {
			String rowNames = rowNamesElement.getTextTrim();
			if ( rowNames.equalsIgnoreCase("T"))
				task.setUseRowNames(true);
		}
		
	}
	private boolean validateOutFile() {
		Map collectorMap = task.getCollectorMap();
		//initialize these to default values, then override if specified
		collectorMap.put(PostProcessTask.PREPEND, "");
		collectorMap.put(PostProcessTask.APPENDBY, PostProcessTask.ROW);
		collectorMap.put(PostProcessTask.FILENAME, constructOutputFileName(Integer.toString(task.getTaskNumber())));
		outFileElement = taskElement.element(PostProcessTask.OUTFILE);
		if ( outFileElement == null ) {
			return true;
		}
		// prepend is a pattern string to determine where replicate number is located
		Element prepend = outFileElement.element(PostProcessTask.PREPEND);
		if ( prepend != null )
			collectorMap.put(PostProcessTask.PREPEND, prepend.getTextTrim());
		Element appendby = outFileElement.element(PostProcessTask.APPENDBY);
		if ( appendby != null ) {
			String appendbyString = appendby.getTextTrim();
			if ( appendbyString.startsWith("C") || appendbyString.startsWith("c"))
				collectorMap.put(PostProcessTask.APPENDBY, PostProcessTask.COLUMN);
			else
				collectorMap.put(PostProcessTask.APPENDBY, PostProcessTask.ROW);
		}
		
		Element fileName = outFileElement.element(PostProcessTask.FILENAME);
		if ( fileName != null )
			collectorMap.put(PostProcessTask.FILENAME, constructOutputFileName(fileName.getTextTrim()));
		
		return true;
	}
	
	/* Used to construct a reasonable output file name from the excursion file name
	 * and the task number if one hasn't been specified.
	 * 
	 */
	private String constructOutputFileName(String suffix) {
		StringBuffer name = new StringBuffer();
		if ( suffix == null )
			name.append(task.getTaskName());
		else
			name.append(suffix);
		name.append(".csv");
		return name.toString();
	}	
	
}

